// Include file for KDTree class

#ifndef __KDTREE__H__
#define __KDTREE__H__



// Node declaration

template <class PtrType>
class KDTreeNode;



// Class declaration

template <class PtrType>
class KDTree {
public:
  // Constructor/destructors
  KDTree(const R3Box& bbox, int position_offset = 0);
  KDTree(const RNArray<PtrType>& points, int position_offset = 0);
  ~KDTree(void);

  // Property functions
  const R3Box& BBox(void) const;
  int NNodes(void) const;

  // Search for closest to a point
  PtrType FindClosest(PtrType point, RNLength min_distance = RN_EPSILON, RNLength max_distance = RN_INFINITY, RNLength *closest_distance = NULL) const;
  PtrType FindClosest(const R3Point& position, RNLength min_distance = RN_EPSILON, RNLength max_distance = RN_INFINITY, RNLength *closest_distance = NULL) const;

  // Search for all within some distance to a point
  int FindAll(PtrType point, RNLength min_distance, RNLength max_distance, RNArray<PtrType>& points) const;
  int FindAll(const R3Point& position, RNLength min_distance, RNLength max_distance, RNArray<PtrType>& points) const;

public:
  // Internal search functions
  void FindClosest(KDTreeNode<PtrType> *node, const R3Box& node_box, const R3Point& position, 
    RNLength min_distance, RNLength max_distance, PtrType& closest_point, RNLength& closest_distance) const;
  void FindAll(KDTreeNode<PtrType> *node, const R3Box& node_box, const R3Point& position, 
    RNLength min_distance, RNLength max_distance, RNArray<PtrType>& points) const;

  // Internal manipulation functions
  void InsertPoints(KDTreeNode<PtrType> *node, const R3Box& node_box, PtrType *points, int npoints);
  int PartitionPoints(PtrType *points, int npoints, RNDimension dim, int imin, int imax);

  // Internal point extraction function
  const R3Point& Position(PtrType point) const
    { return *((R3Point *) ((unsigned char *) point + position_offset)); } ;

  // Internal visualization functions
  void Outline(KDTreeNode<PtrType> *node, const R3Box& bbox) const;
  void Outline(void) const;

public:
  // Internal data
  R3Box bbox;
  int position_offset;
  KDTreeNode<PtrType> *root;
  int nnodes;
};



// Include templated definitions

#include "KDTree.I"


#endif
